#!/usr/bin/python

DOCUMENTATION = r'''
---
module: decort_osimage

description: See L(Module Documentation,https://repository.basistech.ru/BASIS/decort-ansible/wiki/Home).
'''

from ansible.module_utils.basic import AnsibleModule
from ansible.module_utils.basic import env_fallback

from ansible.module_utils.decort_utils import *


class decort_osimage(DecortController):
    def __init__(self,amodule):
        super(decort_osimage, self).__init__(amodule)

        self.validated_image_id = 0
        self.validated_virt_image_id = 0
        self.validated_image_name = amodule.params['image_name']
        self.validated_virt_image_name = None
        self.validated_virt_image_id = amodule.params['virt_id']
        if amodule.params['account_name']:
            self.validated_account_id, _ = self.account_find(amodule.params['account_name'])
        else:
            self.validated_account_id = amodule.params['account_Id']
        
        if self.validated_account_id == 0:
            # we failed either to find or access the specified account - fail the module
            self.result['failed'] = True
            self.result['changed'] = False
            self.result['msg'] = ("Cannot find account '{}'").format(amodule.params['account_name'])
            amodule.fail_json(**self.result)


        if amodule.params['virt_id'] != 0 and amodule.params['virt_name']:
            self.validated_virt_image_id, image_facts =\
                self.decort_virt_image_find(amodule)
            if (self.validated_virt_image_id and
                    amodule.params['virt_name'] != image_facts['name']):
                self.decort_virt_image_rename(amodule)
                self.result['msg'] = 'Virtual image renamed successfully'
        elif amodule.params['image_id'] != 0 and amodule.params['image_name']:
            self.validated_image_id, image_facts = self.decort_image_find(amodule)
            if (self.validated_image_id and
                    amodule.params['image_name'] != image_facts['name']):
                decort_osimage.decort_image_rename(self,amodule)
                self.result['msg'] = ("Image renamed successfully")



    def decort_image_find(self, amodule):
        # function that finds the OS image
        image_id, image_facts = self.image_find(image_id=amodule.params['image_id'], image_name=self.validated_image_name,
                                                account_id=self.validated_account_id, rg_id=0,
                                                sepid=amodule.params['sep_id'],
                                                pool=amodule.params['pool'])
        return image_id, image_facts

    def decort_virt_image_find(self, amodule):
        # function that finds a virtual image
        image_id, image_facts = self.virt_image_find(image_id=amodule.params['virt_id'],
                                                    account_id=self.validated_account_id, rg_id=0,
                                                    sepid=amodule.params['sep_id'],
                                                    virt_name=amodule.params['virt_name'],
                                                    pool=amodule.params['pool'])
        return image_id, image_facts



    def decort_image_create(self,amodule):
        # function that creates OS image
        image_facts = self.image_create(img_name=self.validated_image_name, 
                        url=amodule.params['url'], 
                        gid=amodule.params['gid'], 
                        boottype=amodule.params['boottype'],
                        imagetype=amodule.params['imagetype'],
                        hotresize=amodule.params['hotresize'],
                        username=amodule.params['image_username'],
                        password=amodule.params['image_password'],
                        account_Id=self.validated_account_id,
                        usernameDL=amodule.params['usernameDL'],
                        passwordDL=amodule.params['passwordDL'],
                        sepId=amodule.params['sepId'],
                        poolName=amodule.params['poolName'],
                        drivers=amodule.params['drivers'])
        self.result['changed'] = True
        return image_facts

    def decort_virt_image_link(self,amodule):
        # function that links an OS image to a virtual one
        self.virt_image_link(imageId=self.validated_virt_image_id, targetId=self.target_image_id)
        image_id, image_facts = decort_osimage.decort_virt_image_find(self, amodule)
        self.result['facts'] = decort_osimage.decort_osimage_package_facts(image_facts, amodule.check_mode)
        self.result['msg'] = ("Image '{}' linked to virtual image '{}'").format(self.target_image_id,
                                                                                decort_osimage.decort_osimage_package_facts(image_facts)['id'],)
        return image_id, image_facts
    
    def decort_image_delete(self,amodule):
        # function that removes an image
        self.image_delete(imageId=amodule.image_id_delete)
        self.result['changed'] = True
        self.result['msg'] = ("Image '{}' deleted").format(amodule.image_id_delete)

    def decort_virt_image_create(self,amodule):
        # function that creates a virtual image
        image_facts = self.virt_image_create(name=amodule.params['virt_name'], targetId=self.target_image_id)
        image_id, image_facts = decort_osimage.decort_virt_image_find(self, amodule)
        self.result['facts'] = decort_osimage.decort_osimage_package_facts(image_facts, amodule.check_mode)
        return image_id, image_facts
    
    def decort_image_rename(self,amodule):
        # image renaming function
        image_facts = self.image_rename(imageId=self.validated_image_id, name=amodule.params['image_name'])
        self.result['msg'] = ("Image renamed successfully")
        image_id, image_facts = decort_osimage.decort_image_find(self, amodule)
        return image_id, image_facts

    def decort_virt_image_rename(self, amodule):
        image_facts = self.image_rename(imageId=self.validated_virt_image_id,
                                        name=amodule.params['virt_name'])
        self.result['msg'] = ("Virtual image renamed successfully")
        image_id, image_facts = self.decort_virt_image_find(amodule)
        return image_id, image_facts

    def decort_osimage_package_facts(arg_osimage_facts, arg_check_mode=False):
        """Package a dictionary of OS image according to the decort_osimage module specification. This
        dictionary will be returned to the upstream Ansible engine at the completion of the module run.

        @param arg_osimage_facts: dictionary with OS image facts as returned by API call to .../images/list
        @param arg_check_mode: boolean that tells if this Ansible module is run in check mode.

        @return: dictionary with OS image specs populated from arg_osimage_facts.
        """

        ret_dict = dict(id=0,
                        name="none",
                        size=0,
                        type="none",
                        state="CHECK_MODE",                        )

        if arg_check_mode:
            # in check mode return immediately with the default values
            return ret_dict

        if arg_osimage_facts is None:
            # if void facts provided - change state value to ABSENT and return
            ret_dict['state'] = "ABSENT"
            return ret_dict
    
        ret_dict['id'] = arg_osimage_facts['id']
        ret_dict['name'] = arg_osimage_facts['name']
        ret_dict['size'] = arg_osimage_facts['size']
        ret_dict['type'] = arg_osimage_facts['type']
        # ret_dict['arch'] = arg_osimage_facts['architecture']
        ret_dict['sep_id'] = arg_osimage_facts['sepId']
        ret_dict['pool'] = arg_osimage_facts['pool']
        ret_dict['state'] = arg_osimage_facts['status']
        ret_dict['linkto'] = arg_osimage_facts['linkTo']
        ret_dict['accountId'] = arg_osimage_facts['accountId']
        return ret_dict


    def decort_osimage_parameters():
        """Build and return a dictionary of parameters expected by decort_osimage module in a form accepted
        by AnsibleModule utility class."""

        return dict(
            app_id=dict(type='str',
                        required=False,
                        fallback=(env_fallback, ['DECORT_APP_ID'])),
            app_secret=dict(type='str',
                            required=False,
                            fallback=(env_fallback, ['DECORT_APP_SECRET']),
                            no_log=True),
            authenticator=dict(type='str',
                               required=True,
                               choices=['legacy', 'oauth2', 'jwt']),
            controller_url=dict(type='str', required=True),
            jwt=dict(type='str',
                     required=False,
                     fallback=(env_fallback, ['DECORT_JWT']),
                     no_log=True),
            oauth2_url=dict(type='str',
                            required=False,
                            fallback=(env_fallback, ['DECORT_OAUTH2_URL'])),
            password=dict(type='str',
                          required=False,
                          fallback=(env_fallback, ['DECORT_PASSWORD']),
                          no_log=True),
            pool=dict(type='str', required=False, default=""),
            sep_id=dict(type='int', required=False, default=0),
            account_name=dict(type='str', required=False),
            account_Id=dict(type='int', required=False),
            user=dict(type='str',
                      required=False,
                      fallback=(env_fallback, ['DECORT_USER'])),
            verify_ssl=dict(type='bool', required=False, default=True),
            workflow_callback=dict(type='str', required=False),
            workflow_context=dict(type='str', required=False),
            image_name=dict(type='str', required=False),
            image_id=dict(type='int', required=False,default=0),
            virt_id=dict(type='int', required=False, default=0),
            virt_name=dict(type='str', required=False),
            state=dict(type='str',
                   default='present',
                   choices=['absent', 'present']),
            drivers=dict(type='str', required=False, default="KVM_X86"),
            imagetype=dict(type='str', required=False, default="linux"),
            boottype=dict(type='str', required=False, default="uefi"),
            url=dict(type='str', required=False),
            gid=dict(type='int', required=False, default=0),
            sepId=dict(type='int', required=False, default=0),
            poolName=dict(type='str', required=False),
            hotresize=dict(type='bool', required=False, default=False),
            image_username=dict(type='str', required=False),
            image_password=dict(type='str', required=False),
            usernameDL=dict(type='str', required=False),
            passwordDL=dict(type='str', required=False),
        )
        

def main():
    module_parameters = decort_osimage.decort_osimage_parameters()

    amodule = AnsibleModule(argument_spec=module_parameters,
                            supports_check_mode=True,
                            mutually_exclusive=[
                                ['oauth2', 'password'],
                                ['password', 'jwt'],
                                ['jwt', 'oauth2'],
                            ],
                            required_together=[
                                ['app_id', 'app_secret'],
                                ['user', 'password'],
                            ],
                            )

    decon = decort_osimage(amodule)

    if amodule.params['virt_name'] or amodule.params['virt_id']:

        image_id, image_facts = decort_osimage.decort_virt_image_find(decon, amodule)
        if amodule.params['image_name'] or amodule.params['image_id']:
            decon.target_image_id, _ = decort_osimage.decort_image_find(decon, amodule)
        else:
            decon.target_image_id = 0
        if decort_osimage.decort_osimage_package_facts(image_facts)['id'] > 0:
            decon.result['facts'] = decort_osimage.decort_osimage_package_facts(image_facts, amodule.check_mode)
        decon.validated_virt_image_id = decort_osimage.decort_osimage_package_facts(image_facts)['id']
        decon.validated_virt_image_name = decort_osimage.decort_osimage_package_facts(image_facts)['name']

        if decort_osimage.decort_osimage_package_facts(image_facts)['id'] == 0 and amodule.params['state'] == "present" and decon.target_image_id > 0:
            image_id, image_facts = decort_osimage.decort_virt_image_create(decon,amodule)
            decon.result['msg'] = ("Virtual image '{}' created").format(decort_osimage.decort_osimage_package_facts(image_facts)['id'])
            decon.result['changed'] = True
        elif decort_osimage.decort_osimage_package_facts(image_facts)['id'] == 0 and amodule.params['state'] == "present" and decon.target_image_id == 0:
            decon.result['msg'] = ("Cannot find OS image")
            amodule.fail_json(**decon.result)

        if decon.validated_virt_image_id and decon.target_image_id:
            if decort_osimage.decort_osimage_package_facts(image_facts)['linkto'] != decon.target_image_id:
                decort_osimage.decort_virt_image_link(decon,amodule)
                decon.result['changed'] = True
                amodule.exit_json(**decon.result)

        if decon.validated_virt_image_id > 0 and amodule.params['state'] == "absent":
            decon.result['msg'] = ("Osimage module cannot delete virtual images.")
            decon.result['failed'] = True
            amodule.exit_json(**decon.result)

    elif amodule.params['image_name'] or amodule.params['image_id']:
        image_id, image_facts = decort_osimage.decort_image_find(decon, amodule)
        decon.validated_image_id = decort_osimage.decort_osimage_package_facts(image_facts)['id']
        if decort_osimage.decort_osimage_package_facts(image_facts)['id'] > 0:
            decon.result['facts'] = decort_osimage.decort_osimage_package_facts(image_facts, amodule.check_mode)

        if amodule.params['state'] == "present" and decon.validated_image_id == 0 and amodule.params['image_name'] and amodule.params['url']:
            decort_osimage.decort_image_create(decon,amodule)
            decon.result['changed'] = True
            image_id, image_facts = decort_osimage.decort_image_find(decon, amodule)
            decon.result['msg'] = ("OS image '{}' created").format(decort_osimage.decort_osimage_package_facts(image_facts)['id'])
            decon.result['facts'] = decort_osimage.decort_osimage_package_facts(image_facts, amodule.check_mode)
            decon.validated_image_id = decort_osimage.decort_osimage_package_facts(image_facts)['id']

        elif amodule.params['state'] == "absent" and decon.validated_image_id:
                amodule.image_id_delete = decon.validated_image_id
                decort_osimage.decort_image_delete(decon,amodule)

    if decon.result['failed'] == True:
        # we failed to find the specified image - fail the module
        decon.result['changed'] = False
        amodule.fail_json(**decon.result)

    amodule.exit_json(**decon.result)

    
if __name__ == "__main__":
    main()
